__all__ = [
    "__title__",
    "__summary__",
    "__uri__",
    "__version__",
    "__author__",
    "__email__",
    "__license__",
    "__copyright__",
]

__title__ = "vivarium"
__summary__ = "vivarium is a microsimulation framework built on top of the standard scientific python stack."
__uri__ = "https://github.com/ihmeuw/vivarium"

__version__ = "1.2.0"

__author__ = "The vivarium developers"
__email__ = "vivarium.dev@gmail.com"

__license__ = "BSD-3-Clause"
__copyright__ = f"Copyright 2016-2023 {__author__}"
